@@ -1,7 +1,7 @@
 
 package CPAN::DistnameInfo;
 
-$VERSION = "0.06";
+$VERSION = "0.09";
 use strict;
 
 sub distname_info {
@@ -17,6 +17,10 @@ sub distname_info {
     )+)(.*)
   $/xs or return ($file,undef,undef);
 
+  if ($dist =~ /-undef\z/ and ! length $version) {
+    $dist =~ s/-undef\z//;
+  }
+
   if ($version =~ /^(-[Vv].*)-(\d.*)/) {
    
     # Catch names like Unicode-Collate-Standard-V3_1_1-0.1
@@ -25,6 +29,10 @@ sub distname_info {
     $version = $2;
   }
 
+  # Normalize the Dist.pm-1.23 convention which CGI.pm and
+  # a few others use.
+  $dist =~ s{\.pm$}{};
+
   $version = $1
     if !length $version and $dist =~ s/-(\d+\w)$//;
 
@@ -65,7 +73,7 @@ sub new {
   ($info{filename} = $distfile) =~ s,^(((.*?/)?authors/)?id/)?([A-Z])/(\4[A-Z])/(\5[-A-Z0-9]*)/,,
     and $info{cpanid} = $6;
 
-  if ($distfile =~ m,([^/]+)\.(tar\.g?z|zip|tgz)$,i) { # support more ?
+  if ($distfile =~ m,([^/]+)\.(tar\.(?:g?z|bz2)|zip|tgz)$,i) { # support more ?
     $info{distvname} = $1;
     $info{extension} = $2;
   }
